﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using Infragistics.Olap.Xmla;
using Infragistics.Win;
using Infragistics.Win.AppStyling;

namespace PivotGridDemo
{
    public partial class SampleForm : Form
    {
        #region Private members
        private readonly List<string> _loadedStyleLibraryList = new List<string>(); 
        #endregion Private members

        #region Default Constructor
        public SampleForm()
        {
            InitializeComponent();
            cmboAppStyleLibrary.ValueChanged += cmboAppStyleLibrary_ValueChanged;
            InitializeForm();
        }
        #endregion //Default Constructor

        #region Events

        #region cmboAppStyleLibrary_ValueChanged
        private void cmboAppStyleLibrary_ValueChanged(object sender, EventArgs e)
        {
            LoadStyleLibrary(cmboAppStyleLibrary.SelectedItem.DisplayText, cmboAppStyleLibrary.Value.ToString());
        }
        #endregion cmboAppStyleLibrary_ValueChanged

        #endregion

        #region Methods

        #region InitializeForm
        public void InitializeForm()
        {
            // Add the form's caption as the sample name
            ultraFormManager1.FormStyleSettings.Caption = Properties.Resources.SampleTitle;

            // Increase the font size
            lblSelectTheme.Font = new Font("Microsoft Sans Serif", 10f);

            // Bind the pivot grid to Infragistics sample data. 
            var settings = new XmlaInitialSettings
            {
                ServerUrl = "http://sampledata.infragistics.com/olap/msmdpump.dll",
                Catalog = "Adventure Works DW Standard Edition",
                Cube = "Adventure Works",
                Rows = "[Date].[Calendar]",
                Columns = "[Product].[Category]",
                Measures = "[Measures].[Reseller Sales Amount]"
            };

            // Create an instance of XMLADataSource passing the same initial settings as in pivit grid. 
            var dataSource = new XmlaDataSource(settings);
            ultraPivotGrid1.DataSource = dataSource;

            // Bind the DataSelector to the same datasource as the pivot grid.
            mdxDataSelector1.DataSource = dataSource;

            // Iterate through style library resource files and create keys for each style.
            // These keys will be used to set selected styles for PivotGrid and DataSelector.
            int index = -1;
            foreach (string path in GetStyleLibraryResourceNames())
            {
                if (path.Contains("Metro"))
                {
                    index = cmboAppStyleLibrary.Items.Count;
                }
                string key = path.Replace(".isl", "");
                cmboAppStyleLibrary.Items.Add(path, key.Remove(0, key.LastIndexOf('.') +1));
            }

            // Set the default style using an index.
            if (index >= 0)
            {
                cmboAppStyleLibrary.SelectedIndex = index;
            }
        }
        #endregion InitializeForm

        #region LoadStyleLibrary
        // Load StyleLibrary (isl) file for PivotGrid and DataSelector
        private void LoadStyleLibrary(string styleLibraryKey, string resourcePath)
        {
            if (!_loadedStyleLibraryList.Contains(styleLibraryKey))
            {
                using (Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(resourcePath))
                {
                    StyleManager.Load(stream, false, styleLibraryKey);
                }
                _loadedStyleLibraryList.Add(styleLibraryKey);
            }

            SetStyleLibraryName(ultraPivotGrid1, styleLibraryKey, false);
            SetStyleLibraryName(mdxDataSelector1, styleLibraryKey, true);
        }
        #endregion LoadStyleLibrary

        #region GetStyleLibraryResourceNames
        // Gets an array of  style library resource names.
        internal static string[] GetStyleLibraryResourceNames()
        {
            var resourceStrings = new List<string>(Assembly.GetExecutingAssembly().GetManifestResourceNames());
            resourceStrings.Sort();

            return resourceStrings.FindAll(i => i.EndsWith(".isl")).ToArray();
        }
        #endregion GetStyleLibraryResourceNames

        #region SetStyleLibraryName
        private void SetStyleLibraryName(Control control, string styleLibraryName, bool recursive)
        {
            if (recursive)
            {
                foreach (Control child in control.Controls)
                {
                    SetStyleLibraryName(child, styleLibraryName, true);
                }
            }

            var ultraControl = control as UltraControlBase;
            if (ultraControl != null)
            {
                ultraControl.StyleLibraryName = styleLibraryName;
            }
        }
        #endregion SetStyleLibraryName

        #endregion Methods
    }
}
